"""
FastAPI Web Application for Hyperliquid Trading Bot

Provides web interface for:
- Dashboard with real-time trading data
- Analytics with comprehensive trading statistics
- Copy trading management and account analysis
"""

import logging
import os
from pathlib import Path
from fastapi import FastAPI, Request, HTTPException
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
from fastapi.middleware.cors import CORSMiddleware
from fastapi.responses import HTMLResponse

from src.config.config import Config
from .dependencies import init_dependencies
from .routers import dashboard_router, analytics_router, copy_trading_router

logger = logging.getLogger(__name__)

# Get the web module directory
WEB_DIR = Path(__file__).parent
TEMPLATES_DIR = WEB_DIR / "templates"
STATIC_DIR = WEB_DIR / "static"

# Ensure directories exist
TEMPLATES_DIR.mkdir(exist_ok=True)
STATIC_DIR.mkdir(exist_ok=True)
(STATIC_DIR / "css").mkdir(exist_ok=True)
(STATIC_DIR / "js").mkdir(exist_ok=True)
(STATIC_DIR / "img").mkdir(exist_ok=True)


def create_app() -> FastAPI:
    """Create and configure the FastAPI application."""
    
    app = FastAPI(
        title="Hyperliquid Trading Bot Web Interface",
        description="Web interface for managing and monitoring Hyperliquid trading operations",
        version="1.0.0",
        docs_url="/docs" if Config.WEB_ENABLED else None,
        redoc_url="/redoc" if Config.WEB_ENABLED else None
    )
    
    # Add CORS middleware
    if Config.WEB_CORS_ORIGINS:
        origins = [origin.strip() for origin in Config.WEB_CORS_ORIGINS.split(",")]
        app.add_middleware(
            CORSMiddleware,
            allow_origins=origins,
            allow_credentials=True,
            allow_methods=["*"],
            allow_headers=["*"],
        )
    
    # Mount static files
    app.mount("/static", StaticFiles(directory=str(STATIC_DIR)), name="static")
    
    # Initialize templates
    templates = Jinja2Templates(directory=str(TEMPLATES_DIR))
    
    # Include routers
    app.include_router(dashboard_router, prefix="/api")
    app.include_router(analytics_router, prefix="/api")
    app.include_router(copy_trading_router, prefix="/api")
    
    # Root route - Dashboard
    @app.get("/", response_class=HTMLResponse)
    async def dashboard_page(request: Request):
        """Serve the main dashboard page."""
        try:
            return templates.TemplateResponse(
                "dashboard.html",
                {
                    "request": request,
                    "title": "Dashboard",
                    "config": {
                        "api_base": "",
                        "default_token": Config.DEFAULT_TRADING_TOKEN,
                        "testnet": Config.HYPERLIQUID_TESTNET
                    }
                }
            )
        except Exception as e:
            logger.error(f"Error serving dashboard: {e}")
            raise HTTPException(status_code=500, detail="Error loading dashboard")
    
    # Analytics page
    @app.get("/analytics", response_class=HTMLResponse)
    async def analytics_page(request: Request):
        """Serve the analytics page."""
        try:
            return templates.TemplateResponse(
                "analytics.html",
                {
                    "request": request,
                    "title": "Analytics",
                    "config": {
                        "api_base": "",
                        "default_token": Config.DEFAULT_TRADING_TOKEN,
                        "testnet": Config.HYPERLIQUID_TESTNET
                    }
                }
            )
        except Exception as e:
            logger.error(f"Error serving analytics: {e}")
            raise HTTPException(status_code=500, detail="Error loading analytics")
    
    # Copy trading page
    @app.get("/copy-trading", response_class=HTMLResponse)
    async def copy_trading_page(request: Request):
        """Serve the copy trading management page."""
        try:
            return templates.TemplateResponse(
                "copy_trading.html",
                {
                    "request": request,
                    "title": "Copy Trading",
                    "config": {
                        "api_base": "",
                        "default_token": Config.DEFAULT_TRADING_TOKEN,
                        "testnet": Config.HYPERLIQUID_TESTNET
                    }
                }
            )
        except Exception as e:
            logger.error(f"Error serving copy trading: {e}")
            raise HTTPException(status_code=500, detail="Error loading copy trading")
    
    # Health check endpoint
    @app.get("/health")
    async def health_check():
        """Health check endpoint."""
        return {
            "status": "healthy",
            "service": "hyperliquid-bot-web"
        }
    
    logger.info("✅ FastAPI application created successfully")
    return app


def initialize_app_dependencies(trading_engine, monitoring_coordinator):
    """Initialize the application with dependencies."""
    init_dependencies(trading_engine, monitoring_coordinator)
    logger.info("✅ App dependencies initialized") 