"""
FastAPI Dependencies for Web UI

Handles authentication, database access, and other shared resources.
"""

import logging
from typing import Optional, Annotated
from fastapi import Depends, HTTPException, status, Header
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials

from src.config.config import Config
from src.trading.trading_engine import TradingEngine
from src.monitoring.monitoring_coordinator import MonitoringCoordinator
from src.clients.hyperliquid_client import HyperliquidClient
from src.stats.trading_stats import TradingStats

logger = logging.getLogger(__name__)

# Security scheme
security = HTTPBearer()

# Global instances (will be initialized in app.py)
trading_engine: Optional[TradingEngine] = None
monitoring_coordinator: Optional[MonitoringCoordinator] = None


def init_dependencies(engine: TradingEngine, coordinator: MonitoringCoordinator):
    """Initialize global dependencies."""
    global trading_engine, monitoring_coordinator
    trading_engine = engine
    monitoring_coordinator = coordinator
    logger.info("✅ Web dependencies initialized")


async def verify_api_key(
    credentials: Annotated[HTTPAuthorizationCredentials, Depends(security)]
) -> str:
    """Verify API key authentication."""
    if not Config.WEB_API_KEY:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail="API key not configured"
        )
    
    if credentials.credentials != Config.WEB_API_KEY:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Invalid API key",
            headers={"WWW-Authenticate": "Bearer"},
        )
    
    return credentials.credentials


async def get_trading_engine() -> TradingEngine:
    """Get the trading engine instance."""
    if trading_engine is None:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail="Trading engine not initialized"
        )
    return trading_engine


async def get_monitoring_coordinator() -> MonitoringCoordinator:
    """Get the monitoring coordinator instance."""
    if monitoring_coordinator is None:
        raise HTTPException(
            status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
            detail="Monitoring coordinator not initialized"
        )
    return monitoring_coordinator


async def get_client() -> HyperliquidClient:
    """Get the Hyperliquid client instance."""
    engine = await get_trading_engine()
    return engine.client


async def get_stats() -> TradingStats:
    """Get the TradingStats instance."""
    engine = await get_trading_engine()
    return engine.get_stats()


# Dependency aliases for convenience
AuthenticatedUser = Annotated[str, Depends(verify_api_key)]
TradingEngineInstance = Annotated[TradingEngine, Depends(get_trading_engine)]
MonitoringInstance = Annotated[MonitoringCoordinator, Depends(get_monitoring_coordinator)] 