"""
Dashboard API Router

Provides endpoints for dashboard overview data including account summary,
positions, recent trades, and key performance metrics.
"""

import logging
from datetime import datetime, timezone
from typing import Dict, List, Any, Optional
from fastapi import APIRouter, HTTPException, Depends
from pydantic import BaseModel

from ..dependencies import AuthenticatedUser, TradingEngineInstance, MonitoringInstance, get_client, get_stats
from src.clients.hyperliquid_client import HyperliquidClient
from src.stats.trading_stats import TradingStats

logger = logging.getLogger(__name__)

router = APIRouter(prefix="/dashboard", tags=["dashboard"])


class DashboardSummary(BaseModel):
    """Dashboard summary response model."""
    balance: float
    total_pnl: float
    win_rate: float
    open_positions: int
    total_return_pct: float
    total_trades: int
    profit_factor: float
    max_drawdown_pct: float


class Position(BaseModel):
    """Position response model."""
    symbol: str
    side: str
    size: float
    entry_price: float
    mark_price: float
    unrealized_pnl: float
    unrealized_pnl_pct: float


class RecentTrade(BaseModel):
    """Recent trade response model."""
    symbol: str
    side: str
    size: float
    price: float
    realized_pnl: float
    timestamp: str


class MarketData(BaseModel):
    symbol: str
    price: float
    change_24h: float
    high_24h: float
    low_24h: float


@router.get("/summary", response_model=DashboardSummary)
async def get_dashboard_summary(
    client: HyperliquidClient = Depends(get_client),
    stats: TradingStats = Depends(get_stats)
):
    """Get dashboard summary data."""
    try:
        # Get current balance (fix: use get_balance instead of get_account_info)
        balance_data = client.get_balance()
        current_balance = 0.0
        if balance_data and 'USDC' in balance_data:
            current_balance = float(balance_data['USDC']['total'])
        
        # Get performance stats
        perf_stats = stats.get_performance_stats()
        
        return DashboardSummary(
            balance=current_balance,
            total_pnl=perf_stats.get('total_pnl', 0.0),
            win_rate=perf_stats.get('win_rate', 0.0),
            open_positions=stats._get_open_positions_count_from_db(),
            total_return_pct=perf_stats.get('total_pnl', 0.0) / max(perf_stats.get('initial_balance', 1.0), 1.0) * 100,
            total_trades=perf_stats.get('total_trades', 0),
            profit_factor=perf_stats.get('profit_factor', 0.0),
            max_drawdown_pct=abs(perf_stats.get('max_drawdown_pct', 0.0))
        )
    except Exception as e:
        logger.warning(f"Could not get current balance: {e}")
        # Return stats without balance
        perf_stats = stats.get_performance_stats()
        return DashboardSummary(
            balance=0.0,
            total_pnl=perf_stats.get('total_pnl', 0.0),
            win_rate=perf_stats.get('win_rate', 0.0),
            open_positions=stats._get_open_positions_count_from_db(),
            total_return_pct=perf_stats.get('total_pnl', 0.0) / max(perf_stats.get('initial_balance', 1.0), 1.0) * 100,
            total_trades=perf_stats.get('total_trades', 0),
            profit_factor=perf_stats.get('profit_factor', 0.0),
            max_drawdown_pct=abs(perf_stats.get('max_drawdown_pct', 0.0))
        )


@router.get("/positions", response_model=List[Position])
async def get_positions(
    client: HyperliquidClient = Depends(get_client),
    stats: TradingStats = Depends(get_stats)
):
    """Get current positions."""
    try:
        # Fix: get_positions is not async, don't await it
        positions = client.get_positions()
        
        if not positions:
            return []
        
        result = []
        for pos in positions:
            if float(pos.get('contracts', 0)) != 0:  # Only open positions
                unrealized_pnl = float(pos.get('unrealizedPnl', 0))
                position_value = abs(float(pos.get('notional', 0)))
                unrealized_pnl_pct = (unrealized_pnl / position_value * 100) if position_value > 0 else 0.0
                
                result.append(Position(
                    symbol=pos.get('symbol', ''),
                    side='long' if float(pos.get('contracts', 0)) > 0 else 'short',
                    size=abs(float(pos.get('contracts', 0))),
                    entry_price=float(pos.get('entryPrice', 0)),
                    mark_price=float(pos.get('markPrice', 0)),
                    unrealized_pnl=unrealized_pnl,
                    unrealized_pnl_pct=unrealized_pnl_pct
                ))
        
        return result
    
    except Exception as e:
        logger.error(f"Error getting positions: {e}")
        return []


@router.get("/recent-trades", response_model=List[RecentTrade])
async def get_recent_trades(
    stats: TradingStats = Depends(get_stats)
):
    """Get recent completed trades."""
    try:
        # Fix: Query trades that have been closed (position_closed status)
        # and use correct column names from the trades table
        query = """
        SELECT symbol, side, amount, price, realized_pnl, position_closed_at
        FROM trades 
        WHERE status = 'position_closed' 
        ORDER BY position_closed_at DESC 
        LIMIT 10
        """
        
        trades = stats.db_manager._fetch_query(query)
        
        result = []
        for trade in trades:
            result.append(RecentTrade(
                symbol=trade['symbol'],
                side=trade['side'],
                size=float(trade['amount'] or 0),
                price=float(trade['price'] or 0),
                realized_pnl=float(trade['realized_pnl'] or 0),
                timestamp=trade['position_closed_at'] or ''
            ))
        
        return result
    
    except Exception as e:
        logger.error(f"Error getting recent trades: {e}")
        return []


@router.get("/market-overview", response_model=List[MarketData])
async def get_market_overview(
    client: HyperliquidClient = Depends(get_client)
):
    """Get market overview for major symbols."""
    symbols = ['BTC/USDC:USDC', 'ETH/USDC:USDC', 'SOL/USDC:USDC', 'DOGE/USDC:USDC']
    result = []
    
    for symbol in symbols:
        try:
            # Fix: get_market_data is not async, don't await it
            market_data = client.get_market_data(symbol)
            
            if market_data and 'ticker' in market_data:
                ticker = market_data['ticker']
                
                # Debug: Log the ticker structure
                logger.debug(f"Ticker data for {symbol}: {ticker}")
                
                # Extract price data with robust None handling
                current_price = 0.0
                high_24h = 0.0
                low_24h = 0.0
                open_24h = 0.0
                
                # Get current price (required)
                if ticker.get('last') is not None:
                    current_price = float(ticker['last'])
                elif ticker.get('close') is not None:
                    current_price = float(ticker['close'])
                else:
                    logger.warning(f"No current price found for {symbol}")
                    continue
                
                # Get high/low with fallbacks
                if ticker.get('high') is not None:
                    high_24h = float(ticker['high'])
                else:
                    high_24h = current_price
                    
                if ticker.get('low') is not None:
                    low_24h = float(ticker['low'])
                else:
                    low_24h = current_price
                    
                if ticker.get('open') is not None:
                    open_24h = float(ticker['open'])
                else:
                    open_24h = current_price
                
                # Calculate 24h change percentage
                change_24h = 0.0
                if open_24h > 0 and current_price != open_24h:
                    change_24h = ((current_price - open_24h) / open_24h * 100)
                
                # Use symbol display name (just the base asset)
                display_symbol = symbol.split('/')[0]
                
                result.append(MarketData(
                    symbol=display_symbol,
                    price=current_price,
                    change_24h=change_24h,
                    high_24h=high_24h,
                    low_24h=low_24h
                ))
                
                logger.debug(f"Added market data for {display_symbol}: price={current_price}, change={change_24h:.2f}%")
            else:
                logger.warning(f"Could not get market data for {symbol}: no ticker data")
                
        except Exception as e:
            logger.warning(f"Could not get market data for {symbol}: {e}")
    
    return result 