/**
 * Main Application JavaScript
 * 
 * Contains global utilities, dashboard functionality, and initialization code.
 */

// Global utilities
window.utils = {
    formatCurrency: function(value, decimals = 2) {
        return new Intl.NumberFormat('en-US', {
            style: 'currency',
            currency: 'USD',
            minimumFractionDigits: decimals,
            maximumFractionDigits: decimals
        }).format(value);
    },
    
    formatPercent: function(value, decimals = 2) {
        return (value > 0 ? '+' : '') + value.toFixed(decimals) + '%';
    },
    
    formatTime: function(isoString) {
        if (!isoString) return '--';
        return new Date(isoString).toLocaleTimeString();
    },
    
    formatDate: function(isoString) {
        if (!isoString) return '--';
        return new Date(isoString).toLocaleDateString();
    },
    
    showToast: function(message, type = 'info') {
        const container = document.getElementById('toast-container');
        const toast = document.createElement('div');
        const bgColor = {
            'success': 'bg-green-500',
            'error': 'bg-red-500',
            'warning': 'bg-yellow-500',
            'info': 'bg-blue-500'
        }[type] || 'bg-blue-500';
        
        toast.className = `${bgColor} text-white px-4 py-3 rounded-lg shadow-lg mb-3 transform transition-all duration-300`;
        toast.innerHTML = `
            <div class="flex items-center">
                <span>${message}</span>
                <button onclick="this.parentElement.parentElement.remove()" class="ml-4 text-white hover:text-gray-200">
                    <i class="fas fa-times"></i>
                </button>
            </div>
        `;
        
        container.appendChild(toast);
        
        // Auto remove after 5 seconds
        setTimeout(() => {
            if (toast.parentElement) {
                toast.remove();
            }
        }, 5000);
    },
    
    makeApiRequest: async function(endpoint, options = {}) {
        const config = window.APP_CONFIG || {};
        const url = (config.apiBase || '') + endpoint;
        
        const defaultOptions = {
            headers: {
                'Content-Type': 'application/json',
                'Authorization': `Bearer ${localStorage.getItem('apiKey') || ''}`
            }
        };
        
        const finalOptions = { ...defaultOptions, ...options };
        if (finalOptions.body && typeof finalOptions.body === 'object') {
            finalOptions.body = JSON.stringify(finalOptions.body);
        }
        
        try {
            const response = await fetch(url, finalOptions);
            
            if (!response.ok) {
                throw new Error(`HTTP ${response.status}: ${response.statusText}`);
            }
            
            return await response.json();
        } catch (error) {
            console.error('API Request failed:', error);
            this.showToast(`API Error: ${error.message}`, 'error');
            throw error;
        }
    }
};

// Dashboard functionality
window.dashboard = {
    data: {
        summary: null,
        positions: [],
        recentTrades: [],
        marketOverview: null,
        lastUpdate: null
    },
    
    async loadAll() {
        try {
            const [summary, positions, recentTrades, marketOverview] = await Promise.all([
                utils.makeApiRequest('/api/dashboard/summary'),
                utils.makeApiRequest('/api/dashboard/positions'),
                utils.makeApiRequest('/api/dashboard/recent-trades'),
                utils.makeApiRequest('/api/dashboard/market-overview')
            ]);
            
            this.data = {
                summary,
                positions,
                recentTrades,
                marketOverview,
                lastUpdate: new Date().toISOString()
            };
            
            this.render();
            
        } catch (error) {
            console.error('Failed to load dashboard data:', error);
            utils.showToast('Failed to load dashboard data', 'error');
        }
    },
    
    render() {
        this.renderSummary();
        this.renderPositions();
        this.renderRecentTrades();
        this.renderMarketOverview();
    },
    
    renderSummary() {
        const summary = this.data.summary;
        if (!summary) return;
        
        const summaryCards = document.querySelectorAll('[data-summary]');
        summaryCards.forEach(card => {
            const field = card.dataset.summary;
            const element = card.querySelector('[data-value]');
            
            if (element && summary[field] !== undefined) {
                let value = summary[field];
                
                // Map field names to match new API response
                const fieldMap = {
                    'account_balance': 'balance',
                    'open_positions_count': 'open_positions'
                };
                
                const mappedField = fieldMap[field] || field;
                if (summary[mappedField] !== undefined) {
                    value = summary[mappedField];
                }
                
                // Format based on field type
                if (field.includes('pnl') || field.includes('balance')) {
                    value = utils.formatCurrency(value);
                    // Add color classes for P&L
                    if (field.includes('pnl')) {
                        element.className = value >= 0 ? 'text-profit' : 'text-loss';
                    }
                } else if (field.includes('pct') || field.includes('rate')) {
                    value = utils.formatPercent(value);
                    // Add color classes for percentages
                    element.className = value >= 0 ? 'text-profit' : 'text-loss';
                } else if (field.includes('count') || field.includes('trades') || field.includes('positions')) {
                    value = value.toLocaleString();
                }
                
                element.textContent = value;
            }
        });
    },
    
    renderPositions() {
        const container = document.getElementById('positions-container');
        if (!container) return;
        
        const positions = this.data.positions;
        
        if (!positions || positions.length === 0) {
            container.innerHTML = `
                <div class="text-center py-8 text-gray-500">
                    <i class="fas fa-chart-line text-4xl mb-4"></i>
                    <p>No open positions</p>
                </div>
            `;
            return;
        }
        
        container.innerHTML = positions.map(position => `
            <div class="metric-card">
                <div class="flex justify-between items-start mb-2">
                    <h3 class="font-semibold text-lg">${position.symbol}</h3>
                    <span class="px-2 py-1 rounded text-xs font-medium ${
                        position.side === 'long' ? 'bg-green-100 text-green-800' : 'bg-red-100 text-red-800'
                    }">
                        ${position.side.toUpperCase()}
                    </span>
                </div>
                <div class="space-y-1 text-sm">
                    <div class="flex justify-between">
                        <span class="text-gray-600">Size:</span>
                        <span class="font-medium">${position.size.toFixed(6)}</span>
                    </div>
                    <div class="flex justify-between">
                        <span class="text-gray-600">Entry:</span>
                        <span class="font-medium">${utils.formatCurrency(position.entry_price)}</span>
                    </div>
                    <div class="flex justify-between">
                        <span class="text-gray-600">Mark:</span>
                        <span class="font-medium">${utils.formatCurrency(position.mark_price)}</span>
                    </div>
                    <div class="flex justify-between">
                        <span class="text-gray-600">P&L:</span>
                        <span class="${position.unrealized_pnl >= 0 ? 'text-profit' : 'text-loss'}">
                            ${utils.formatCurrency(position.unrealized_pnl)} (${utils.formatPercent(position.unrealized_pnl_pct)})
                        </span>
                    </div>
                </div>
            </div>
        `).join('');
    },
    
    renderRecentTrades() {
        const container = document.getElementById('recent-trades-container');
        if (!container) return;
        
        const trades = this.data.recentTrades;
        
        if (!trades || trades.length === 0) {
            container.innerHTML = `
                <div class="text-center py-8 text-gray-500">
                    <i class="fas fa-history text-4xl mb-4"></i>
                    <p>No recent trades</p>
                </div>
            `;
            return;
        }
        
        container.innerHTML = trades.map(trade => `
            <div class="flex justify-between items-center py-3 border-b border-gray-100 last:border-b-0">
                <div class="flex items-center space-x-3">
                    <span class="px-2 py-1 rounded text-xs font-medium ${
                        trade.side === 'buy' ? 'bg-green-100 text-green-800' : 'bg-red-100 text-red-800'
                    }">
                        ${trade.side.toUpperCase()}
                    </span>
                    <span class="font-medium">${trade.symbol}</span>
                    <span class="text-sm text-gray-600">${trade.size.toFixed(6)}</span>
                </div>
                <div class="text-right">
                    <div class="${trade.realized_pnl >= 0 ? 'text-profit' : 'text-loss'} font-medium">
                        ${utils.formatCurrency(trade.realized_pnl)}
                    </div>
                    <div class="text-xs text-gray-500">
                        ${utils.formatTime(trade.timestamp)}
                    </div>
                </div>
            </div>
        `).join('');
    },
    
    renderMarketOverview() {
        const container = document.getElementById('market-overview-container');
        if (!container) return;
        
        const market = this.data.marketOverview;
        if (!market || !Array.isArray(market)) return;
        
        container.innerHTML = market.map(data => `
            <div class="metric-card">
                <div class="text-center">
                    <h3 class="font-semibold text-lg mb-1">${data.symbol}</h3>
                    <div class="text-xl font-bold">${utils.formatCurrency(data.price)}</div>
                    <div class="${data.change_24h >= 0 ? 'text-profit' : 'text-loss'} text-sm">
                        ${utils.formatPercent(data.change_24h)} 24h
                    </div>
                    <div class="text-xs text-gray-500 mt-1">
                        H: ${utils.formatCurrency(data.high_24h)} / L: ${utils.formatCurrency(data.low_24h)}
                    </div>
                </div>
            </div>
        `).join('');
    }
};

// API Key management
window.auth = {
    setApiKey(key) {
        localStorage.setItem('apiKey', key);
    },
    
    getApiKey() {
        return localStorage.getItem('apiKey');
    },
    
    clearApiKey() {
        localStorage.removeItem('apiKey');
    },
    
    isAuthenticated() {
        return !!this.getApiKey();
    }
};

// Update last update time
function updateLastUpdateTime() {
    document.getElementById('last-update').textContent = 
        'Last update: ' + new Date().toLocaleTimeString();
}

// Auto-refresh functionality
let autoRefreshInterval = null;

function startAutoRefresh(intervalMs = 30000) {
    if (autoRefreshInterval) {
        clearInterval(autoRefreshInterval);
    }
    
    autoRefreshInterval = setInterval(() => {
        if (window.dashboard && typeof window.dashboard.loadAll === 'function') {
            dashboard.loadAll();
        }
        updateLastUpdateTime();
    }, intervalMs);
}

function stopAutoRefresh() {
    if (autoRefreshInterval) {
        clearInterval(autoRefreshInterval);
        autoRefreshInterval = null;
    }
}

// Initialize application
document.addEventListener('DOMContentLoaded', function() {
    console.log('Hyperliquid Trading Bot Web UI initialized');
    
    // Update time display
    updateLastUpdateTime();
    
    // Check if we're on the dashboard page
    if (typeof window.dashboard !== 'undefined') {
        // Load initial data
        dashboard.loadAll();
        
        // Start auto-refresh
        startAutoRefresh(30000); // 30 seconds
        
        // Add refresh button handlers
        const refreshButton = document.getElementById('refresh-dashboard');
        if (refreshButton) {
            refreshButton.addEventListener('click', () => {
                dashboard.loadAll();
            });
        }
    }
    
    // Handle API key input
    const apiKeyInput = document.getElementById('api-key-input');
    if (apiKeyInput) {
        const savedKey = auth.getApiKey();
        if (savedKey) {
            apiKeyInput.value = savedKey;
        }
        
        apiKeyInput.addEventListener('change', (e) => {
            auth.setApiKey(e.target.value);
        });
    }
}); 